package com.uc_mobileapps.examples.vaadin.forms;

import com.uc_mobileapps.examples.vaadin.dao.CustomerRepository;
import com.uc_mobileapps.examples.vaadin.entities.Customer;
import com.uc_mobileapps.examples.vaadin.views.SimpleGrid;
import com.vaadin.flow.component.ClickEvent;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.checkbox.Checkbox;
import com.vaadin.flow.component.datepicker.DatePicker;
import com.vaadin.flow.component.formlayout.FormLayout;
import com.vaadin.flow.component.textfield.PasswordField;
import com.vaadin.flow.component.textfield.TextField;
import com.vaadin.flow.data.binder.ValidationResult;
import com.vaadin.flow.spring.annotation.SpringComponent;
import com.weebmeister.seife.anno.SeifeBinding;
import com.weebmeister.seife.anno.SeifeForm;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;

import static org.springframework.beans.factory.config.ConfigurableBeanFactory.SCOPE_PROTOTYPE;

import com.vaadin.flow.data.binder.Binder;
import com.vaadin.flow.data.binder.Binder.Binding;
import com.vaadin.flow.data.binder.Binder.BindingBuilder;

import javax.annotation.PostConstruct;
import java.time.LocalDate;
import java.time.ZoneId;

/**
 * Simple customer editor, the three fields are the only things to declare.
 * The generated code is right in the form class, can be read and understood easily and does not rely on reflection.
 * @SeifeBinding takes e.g. mandatory defined at the business object into account everywhere it is used in a form.
 */
@SpringComponent
@Scope(SCOPE_PROTOTYPE)
@SeifeForm(forClass = Customer.class, generatorOptions = {"form.formClass"})
public class CustomerForm extends FormLayout {

    /**
     * The recommended way is to name the fields just as in e.g. {@link Customer}, fields are then bound implicitly.
     */
    @SeifeBinding
    private TextField firstName;

    /**
     * However if necessary it can be explicitly bound via the name.
     */
    @SeifeBinding("lastName")
    private TextField name;

    /**
     * You can navigate fields with the dot-notation, even if they may contain null and are not mandatory.
     */
    @SeifeBinding("address.street")
    private TextField street;

	/**
	 * Seife will generate the constraints on the input defined in the {@link Customer} for you.
	 */
	@SeifeBinding
	private TextField title;

	/**
	 * Modern Java date and time api is supported.
	 */
	@SeifeBinding
    private DatePicker birthday;

	@SeifeBinding
	private PasswordField password;

	/**
	 * The binder is initialized to traverse multiple levels of object references.
	 */
    @SeifeBinding("favoriteProduct.name")
    private TextField favoriteProductName;

	@SeifeBinding
	private Checkbox premium;

	/**
	 * Every binding can be controlled to
	 */
	@SeifeBinding(instantiate = false)
	private Checkbox active;

	private Button save = new Button("Save");

	/**
	 * The vaadin bean binder.
	 */
	private Binder<Customer> binder;

	/**
	 * The Spring customer repository. In real scenarios it is recommended to use a service instead that has the
	 * repository autowired.
	 */
	private CustomerRepository customerRepository;

	@Autowired
    public CustomerForm(CustomerRepository customerRepo) {
		customerRepository = customerRepo;
	}

	public void setCustomer(Customer customer) {
    	binder.setBean(customer);
	}

    private Binder<Customer> getBinder() {
        if (binder == null) {
            binder = new Binder<>(Customer.class);
        }
        return binder;
    }

    @PostConstruct
    public void setup() {

    	// this checkbox instantiate flag is false in the binding, hence no code is generated.
		active = new Checkbox();

		setupFields();

    	// localization can also be achieved in a generic way by customizing the templates
		title.setLabel("Title");
    	add(title);
    	firstName.setLabel("Firstname");
    	add(firstName);
    	name.setLabel("Name");
    	add(name);
    	password.setLabel("Password");
    	add(password);
    	street.setLabel("Street");
    	add(street);
		birthday.setLabel("Birthday");
		add(birthday);
		premium.setLabel("Premium user");
		add(premium);
		active.setLabel("Is active");
		add(active);

    	createBindings();

    	// any bindings can be adjusted before binding
    	builderBirthday.withValidator((birthday, b) ->
				(birthday == null || LocalDate.now(ZoneId.of("UTC")).minusYears(18).isAfter(birthday))
						? ValidationResult.ok() : ValidationResult.error("Customer must be 18 or older"));

    	bindAutogenerated();

    	save.addClickListener(this::save);
    	add(save);
	}

	private void save(ClickEvent<Button> buttonClickEvent) {
		if (getBinder().validate().isOk()) {
			customerRepository.save(binder.getBean());
			getUI().ifPresent(ui -> ui.navigate(SimpleGrid.class));
		}
    }

    // @seife automatically generated:


	@SuppressWarnings("unused")
	private Binding<Customer, String> bindingFirstName;
	@SuppressWarnings("unused")
	private Binding<Customer, String> bindingLastName;
	@SuppressWarnings("unused")
	private Binding<Customer, String> bindingStreet;
	@SuppressWarnings("unused")
	private Binding<Customer, java.time.LocalDate> bindingBirthday;
	@SuppressWarnings("unused")
	private Binding<Customer, String> bindingPassword;
	@SuppressWarnings("unused")
	private Binding<Customer, String> bindingName;
	@SuppressWarnings("unused")
	private Binding<Customer, Boolean> bindingPremium;
	@SuppressWarnings("unused")
	private Binding<Customer, Boolean> bindingActive;
 
	private BindingBuilder<Customer, String> builderFirstName;
	private BindingBuilder<Customer, String> builderLastName;
	private BindingBuilder<Customer, String> builderStreet;
	private BindingBuilder<Customer, java.time.LocalDate> builderBirthday;
	private BindingBuilder<Customer, String> builderPassword;
	private BindingBuilder<Customer, String> builderName;
	private BindingBuilder<Customer, Boolean> builderPremium;
	private BindingBuilder<Customer, Boolean> builderActive;
 
	/**
	 * Call this to setup initial bindings of all annotated attributes. You can customize the bindings before calling {@link #bindAutogenerated()}
	 */
	protected final void createBindings() {
		Binder<Customer> binder = getBinder();
		builderFirstName = binder.forField(firstName);
		builderLastName = binder.forField(name).asRequired();
		builderStreet = binder.forField(street);
		builderBirthday = binder.forField(birthday);
		builderPassword = binder.forField(password);
		builderName = binder.forField(favoriteProductName);
		builderPremium = binder.forField(premium);
		builderActive = binder.forField(active);
 	}

	/**
	 * Call this e.g. from the {@link #bind()} method.
	 */
	protected final void bindAutogenerated() {
		bindingFirstName = builderFirstName.bind("firstName");
		bindingLastName = builderLastName.bind("lastName");
		bindingStreet = builderStreet.bind("address.street");
		bindingBirthday = builderBirthday.bind("birthday");
		bindingPassword = builderPassword.bind("password");
		bindingName = builderName.bind("favoriteProduct.name");
		bindingPremium = builderPremium.bind("premium");
		bindingActive = builderActive.bind("active");
 	}

	/**
	 * Call this e.g. from a @PostConstruct annotated method.
	 */
	protected final void setupFields() {
		firstName = new TextField();
		firstName.setId("firstName");
		name = new TextField();
		name.setId("name");
		street = new TextField();
		street.setId("street");
		title = new TextField();
		title.setId("title");
		birthday = new DatePicker();
		birthday.setId("birthday");
		password = new PasswordField();
		password.setId("password");
		favoriteProductName = new TextField();
		favoriteProductName.setId("favoriteProductName");
		premium = new Checkbox();
		premium.setId("premium");
		active.setId("active");
 	}

	// @seife auto-code end
}
