package com.uc_mobileapps.examples.vaadin.forms;

import com.uc_mobileapps.examples.vaadin.dao.CustomerRepository;
import com.uc_mobileapps.examples.vaadin.entities.Customer;
import com.vaadin.flow.component.checkbox.Checkbox;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.spring.annotation.SpringComponent;
import com.weebmeister.seife.anno.SeifeBinding;
import com.weebmeister.seife.anno.SeifeForm;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;

import java.util.Optional;

import static org.springframework.beans.factory.config.ConfigurableBeanFactory.SCOPE_PROTOTYPE;

/**
 * Simple customer grid, most of the databinding is resolved automatically. The dot notation makes it possible
 * to bind nested properties.
 */
@SpringComponent
@Scope(SCOPE_PROTOTYPE)
@SeifeForm(forClass = Customer.class, generatorOptions = {"grid.gridClass"})
public class CustomerGrid extends Grid<Customer> {

    /**
     * The recommended way is to name the fields just as in e.g. {@link Customer}, fields are then bound implicitly.
     */
    @SeifeBinding
    private Column<Customer> firstName;

    /**
     * However if necessary it can be explicitly bound via the name.
     */
    @SeifeBinding("lastName")
    private Column<Customer> name;

    /**
     * You can navigate fields with the dot-notation, even if they may contain null and are not mandatory.
     */
    @SeifeBinding("address.street")
    private Column<Customer> street;

	/**
	 * Seife will generate the constraints on the input defined in the {@link Customer} for you.
	 */
	@SeifeBinding
	private Column<Customer> title;

	/**
	 * Modern Java date and time api is supported.
	 */
	@SeifeBinding
    private Column<Customer> birthday;

	@SeifeBinding
	private Column<Customer> password;

	/**
	 * The binder is initialized to traverse multiple levels of object references.
	 */
    @SeifeBinding("favoriteProduct.name")
    private Column<Customer> favoriteProductName;

	/**
	 * All boolean fields are automatically mapped to a checkbox, this can be done for any desired type see the
	 * src/main/seife/templates folder and the documentation.
	 */
	@SeifeBinding
	private Column<Customer> premium;

	/**
	 * Every binding can be controlled to
	 */
	@SeifeBinding(instantiate = false)
	private Column<Customer> active;

	/**
	 * The Spring customer repository. In real scenarios it is recommended to use a service instead that has the
	 * repository autowired.
	 */
	private CustomerRepository customerRepository;

	@Autowired
    public CustomerGrid(CustomerRepository customerRepo) {
		customerRepository = customerRepo;
		setupColumns();
		setupHeaders();
	}

	private String getFieldName(String technicalName) {
		// localized header
		return technicalName;
	}

    // @seife automatically generated:
	/** Column key of firstName */
	public static final String COLUMN_FIRST_NAME = "firstName";
	
	/** Column key of lastName */
	public static final String COLUMN_LAST_NAME = "lastName";
	
	/** Column key of address.street */
	public static final String COLUMN_ADDRESS_STREET = "address.street";
	
	/** Column key of birthday */
	public static final String COLUMN_BIRTHDAY = "birthday";
	
	/** Column key of password */
	public static final String COLUMN_PASSWORD = "password";
	
	/** Column key of favoriteProduct.name */
	public static final String COLUMN_FAVORITE_PRODUCT_NAME = "favoriteProduct.name";
	
	/** Column key of premium */
	public static final String COLUMN_PREMIUM = "premium";
	
	/** Column key of active */
	public static final String COLUMN_ACTIVE = "active";
	
	/**
	 * Binds the columns, can be called e.g. from @PostConstruct.
	 */
	protected final void setupColumns() {
		firstName = addColumn(entity -> entity.getFirstName());
		firstName.setKey(COLUMN_FIRST_NAME);
		firstName.setId("firstName");
		name = addColumn(entity -> entity.getLastName());
		name.setKey(COLUMN_LAST_NAME);
		name.setId("name");
		street = addColumn(entity -> Optional.ofNullable(entity.getAddress()).map(p -> p.
getStreet())
.orElse(null));
		street.setKey(COLUMN_ADDRESS_STREET);
		street.setId("street");
		birthday = addColumn(entity -> entity.getBirthday());
		birthday.setKey(COLUMN_BIRTHDAY);
		birthday.setId("birthday");
		password = addColumn(entity -> entity.getPassword());
		password.setKey(COLUMN_PASSWORD);
		password.setId("password");
		favoriteProductName = addColumn(entity -> Optional.ofNullable(entity.getFavoriteProduct()).map(p -> p.
			getName())
			.orElse(null));
		favoriteProductName.setKey(COLUMN_FAVORITE_PRODUCT_NAME);
		favoriteProductName.setId("favoriteProductName");
		premium = addComponentColumn(entity -> {
				Checkbox cb = new Checkbox(entity.isPremium());
				cb.setEnabled(false);
				return cb;
			});
		premium.setKey(COLUMN_PREMIUM);
		premium.setId("premium");
		active = addComponentColumn(entity -> {
				Checkbox cb = new Checkbox(entity.isActive());
				cb.setEnabled(false);
				return cb;
			});
		active.setKey(COLUMN_ACTIVE);
		active.setId("active");
 	}

	/**
	 * Sets up the headers, can be called e.g. from @PostConstruct.
	 */
	protected final void setupHeaders() {
		firstName.setHeader(getFieldName(COLUMN_FIRST_NAME));
		name.setHeader(getFieldName(COLUMN_LAST_NAME));
		street.setHeader(getFieldName(COLUMN_ADDRESS_STREET));
		birthday.setHeader(getFieldName(COLUMN_BIRTHDAY));
		password.setHeader(getFieldName(COLUMN_PASSWORD));
		favoriteProductName.setHeader(getFieldName(COLUMN_FAVORITE_PRODUCT_NAME));
		premium.setHeader(getFieldName(COLUMN_PREMIUM));
		active.setHeader(getFieldName(COLUMN_ACTIVE));
 	}

	// @seife auto-code end
}
